/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.form.business;

import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Date;

import java.util.ArrayList;
import java.util.List;


/**
 * class FormDAO
 */
public final class FormDAO implements IFormDAO
{
    // Constants
    private static final String EMPTY_STRING = "";
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_form ) FROM form_form";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT id_form,title,description, welcome_message," +
        "unavailability_message, requirement_message,workgroup," +
        "id_mailing_list,active_captcha,active_store_adresse," +
        "libelle_validate_button,date_end_disponibility,active,date_creation,limit_number_response,id_recap,active_requirement " +
        " FROM form_form WHERE id_form = ?";
    private static final String SQL_QUERY_INSERT = "INSERT INTO form_form ( id_form,title,description, welcome_message," +
        "unavailability_message,requirement_message,workgroup," +
        "id_mailing_list,active_captcha,active_store_adresse," +
        "libelle_validate_button,date_end_disponibility,active,date_creation,limit_number_response,id_recap,active_requirement) " +
        "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
    private static final String SQL_QUERY_DELETE = "DELETE FROM form_form WHERE id_form = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE form_form SET id_form=?,title=?,description=?, welcome_message=?," +
        "unavailability_message=?, requirement_message=?,workgroup=?," +
        "id_mailing_list=?,active_captcha=?,active_store_adresse=?," +
        "libelle_validate_button=?,date_end_disponibility=?,active=?,limit_number_response=? ,active_requirement=? WHERE id_form=?";
    private static final String SQL_QUERY_SELECT_FORM_BY_FILTER = "SELECT id_form,title,description, welcome_message," +
        "unavailability_message, requirement_message,workgroup," +
        "id_mailing_list,active_captcha,active_store_adresse," +
        "libelle_validate_button,date_end_disponibility,active,date_creation,limit_number_response,id_recap,active_requirement " +
        " FROM form_form WHERE 1 ";
    private static final String SQL_FILTER_WORKGROUP = " AND workgroup = ? ";
    private static final String SQL_FILTER_STATE = " AND active = ? ";
    private static final String SQL_FILTER_STATE_END_DISPONIBILTY_BEFORE_CURRENT_DATE = " AND date_end_disponibility < ? ";
    private static final String SQL_ORDER_BY_DATE_CREATION = " ORDER BY date_creation DESC ";

    /**
     * Generates a new primary key
     *
     * @param plugin the plugin
     * @return The new primary key
     */
    public int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            // if the table is empty
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Insert a new record in the table.
     *
     * @param form instance of the Form to insert
     * @param plugin the plugin
     * @return the new form create
     */
    public synchronized int insert( Form form, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        form.setIdForm( newPrimaryKey( plugin ) );

        daoUtil.setInt( 1, form.getIdForm(  ) );
        daoUtil.setString( 2, form.getTitle(  ) );
        daoUtil.setString( 3, form.getDescription(  ) );
        daoUtil.setString( 4, form.getWelcomeMessage(  ) );
        daoUtil.setString( 5, form.getUnavailabilityMessage(  ) );
        daoUtil.setString( 6, form.getRequirement(  ) );
        daoUtil.setString( 7, form.getWorkgroup(  ) );
        daoUtil.setInt( 8, form.getIdMailingList(  ) );
        daoUtil.setBoolean( 9, form.isActiveCaptcha(  ) );
        daoUtil.setBoolean( 10, form.isActiveStoreAdresse(  ) );
        daoUtil.setString( 11, form.getLibelleValidateButton(  ) );
        daoUtil.setDate( 12,
            ( form.getDateEndDisponibility(  ) != null ) ? new Date( form.getDateEndDisponibility(  ).getTime(  ) ) : null );
        daoUtil.setBoolean( 13, form.isActive(  ) );
        daoUtil.setTimestamp( 14, form.getDateCreation(  ) );
        daoUtil.setBoolean( 15, form.isLimitNumberResponse(  ) );
        daoUtil.setInt( 16, form.getRecap(  ).getIdRecap(  ) );
        daoUtil.setBoolean( 17, form.isActiveRequirement(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return form.getIdForm(  );
    }

    /**
     * Load the data of the Form from the table
     *
     * @param nId The identifier of the form
     * @param plugin the plugin
     * @return the instance of the Form
     */
    public Form load( int nId, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nId );
        daoUtil.executeQuery(  );

        Recap recap = null;
        Form form = null;

        if ( daoUtil.next(  ) )
        {
            form = new Form(  );
            form.setIdForm( daoUtil.getInt( 1 ) );
            form.setTitle( daoUtil.getString( 2 ) );
            form.setDescription( daoUtil.getString( 3 ) );
            form.setWelcomeMessage( daoUtil.getString( 4 ) );
            form.setUnavailabilityMessage( daoUtil.getString( 5 ) );
            form.setRequirement( daoUtil.getString( 6 ) );
            form.setWorkgroup( daoUtil.getString( 7 ) );
            form.setIdMailingList( daoUtil.getInt( 8 ) );
            form.setActiveCaptcha( daoUtil.getBoolean( 9 ) );
            form.setActiveStoreAdresse( daoUtil.getBoolean( 10 ) );
            form.setLibelleValidateButton( daoUtil.getString( 11 ) );
            form.setDateEndDisponibility( daoUtil.getDate( 12 ) );
            form.setActive( daoUtil.getBoolean( 13 ) );
            form.setDateCreation( daoUtil.getTimestamp( 14 ) );
            form.setLimitNumberResponse( daoUtil.getBoolean( 15 ) );
            recap = new Recap(  );
            recap.setIdRecap( daoUtil.getInt( 16 ) );
            form.setRecap( recap );
            form.setActiveRequirement( daoUtil.getBoolean( 17 ) );
        }

        daoUtil.free(  );

        return form;
    }

    /**
     * Delete a record from the table
     *
     * @param nIdForm The identifier of the form
     * @param plugin the plugin
     */
    public void delete( int nIdForm, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, nIdForm );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Update the form in the table
     *
     * @param form instance of the Form object to update
     * @param plugin the plugin
     */
    public void store( Form form, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setInt( 1, form.getIdForm(  ) );
        daoUtil.setString( 2, form.getTitle(  ) );
        daoUtil.setString( 3, form.getDescription(  ) );
        daoUtil.setString( 4, form.getWelcomeMessage(  ) );
        daoUtil.setString( 5, form.getUnavailabilityMessage(  ) );
        daoUtil.setString( 6, form.getRequirement(  ) );
        daoUtil.setString( 7, form.getWorkgroup(  ) );
        daoUtil.setInt( 8, form.getIdMailingList(  ) );
        daoUtil.setBoolean( 9, form.isActiveCaptcha(  ) );
        daoUtil.setBoolean( 10, form.isActiveStoreAdresse(  ) );
        daoUtil.setString( 11, form.getLibelleValidateButton(  ) );
        daoUtil.setDate( 12,
            ( form.getDateEndDisponibility(  ) != null ) ? new Date( form.getDateEndDisponibility(  ).getTime(  ) ) : null );
        daoUtil.setBoolean( 13, form.isActive(  ) );
        daoUtil.setBoolean( 14, form.isLimitNumberResponse(  ) );
        daoUtil.setBoolean( 15, form.isActiveRequirement(  ) );

        daoUtil.setInt( 16, form.getIdForm(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Load the data of all the form who verify the filter and returns them in a  list
     * @param filter the filter
     * @param plugin the plugin
     * @return  the list of form
     */
    public List<Form> selectFormList( FormFilter filter, Plugin plugin )
    {
        List<Form> formList = new ArrayList<Form>(  );
        Form form = null;
        Recap recap = null;
        String strSQL = SQL_QUERY_SELECT_FORM_BY_FILTER;
        strSQL += ( ( filter.containsWorkgroupCriteria(  ) ) ? SQL_FILTER_WORKGROUP : EMPTY_STRING );
        strSQL += ( ( filter.containsIdState(  ) ) ? SQL_FILTER_STATE : EMPTY_STRING );
        strSQL += ( ( filter.containsDateEndDisponibilityBeforeCurrentDate(  ) )
        ? SQL_FILTER_STATE_END_DISPONIBILTY_BEFORE_CURRENT_DATE : EMPTY_STRING );
        strSQL += SQL_ORDER_BY_DATE_CREATION;

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );
        int nIndex = 1;

        if ( filter.containsWorkgroupCriteria(  ) )
        {
            daoUtil.setString( nIndex, filter.getWorkgroup(  ) );
            nIndex++;
        }

        if ( filter.containsIdState(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdState(  ) );
            nIndex++;
        }

        if ( filter.containsDateEndDisponibilityBeforeCurrentDate(  ) )
        {
            daoUtil.setDate( nIndex, new Date( new java.util.Date(  ).getTime(  ) ) );
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            form = new Form(  );
            form.setIdForm( daoUtil.getInt( 1 ) );
            form.setTitle( daoUtil.getString( 2 ) );
            form.setDescription( daoUtil.getString( 3 ) );
            form.setWelcomeMessage( daoUtil.getString( 4 ) );
            form.setUnavailabilityMessage( daoUtil.getString( 5 ) );
            form.setRequirement( daoUtil.getString( 6 ) );
            form.setWorkgroup( daoUtil.getString( 7 ) );
            form.setIdMailingList( daoUtil.getInt( 8 ) );
            form.setActiveCaptcha( daoUtil.getBoolean( 9 ) );
            form.setActiveStoreAdresse( daoUtil.getBoolean( 10 ) );
            form.setLibelleValidateButton( daoUtil.getString( 11 ) );
            form.setDateEndDisponibility( daoUtil.getDate( 12 ) );
            form.setActive( daoUtil.getBoolean( 13 ) );
            form.setDateCreation( daoUtil.getTimestamp( 14 ) );
            form.setLimitNumberResponse( daoUtil.getBoolean( 15 ) );
            recap = new Recap(  );
            recap.setIdRecap( daoUtil.getInt( 16 ) );
            form.setRecap( recap );
            form.setActiveRequirement( daoUtil.getBoolean( 17 ) );

            formList.add( form );
        }

        daoUtil.free(  );

        return formList;
    }

    /**
     * Load the data of all enable form  returns them in a  reference list
     * @param plugin the plugin
     * @return  a  reference list of form
     */
    public ReferenceList getEnableFormList( Plugin plugin )
    {
        ReferenceList listForm = new ReferenceList(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_SELECT_FORM_BY_FILTER + SQL_ORDER_BY_DATE_CREATION, plugin );
        daoUtil.executeQuery(  );

        Form form;

        while ( daoUtil.next(  ) )
        {
            form = new Form(  );
            form.setIdForm( daoUtil.getInt( 1 ) );
            form.setTitle( daoUtil.getString( 2 ) );
            listForm.addItem( form.getIdForm(  ), form.getTitle(  ) );
        }

        daoUtil.free(  );

        return listForm;
    }
}
